from fastapi import APIRouter, Depends, HTTPException
from fastapi.security import OAuth2PasswordBearer
from backend.models.models import User
from backend.schemas.user import UserCreate, UserLogin
from backend.database import get_session
from sqlmodel import Session, select
from backend.utils.hashing import (
    hash_password,
    verify_password,
    decode_token,
    create_access_token,
)


router = APIRouter()

oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/auth/login")


def get_current_user(
    token: str = Depends(oauth2_scheme), session: Session = Depends(get_session)
) -> User:
    user_id = decode_token(token)
    user = session.get(User, user_id)
    if not user:
        raise HTTPException(
            status_code=401, detail="Invalid authentication credentials"
        )
    return user


@router.post("/signup")
def signup(user_data: UserCreate, session: Session = Depends(get_session)):
    existing_user = session.exec(
        select(User).where(User.email == user_data.email)
    ).first()
    if existing_user:
        raise HTTPException(status_code=400, detail="Email already registered")

    hashed_password = hash_password(user_data.password)
    user = User(
        username=user_data.username,
        email=user_data.email,
        password=hashed_password,
        phone_number=user_data.phone_number,
        role="buyer",
    )
    session.add(user)
    session.commit()
    return {"message": "User created successfully"}


@router.post("/login")
def login(user_data: UserLogin, session: Session = Depends(get_session)):
    user = session.exec(select(User).where(User.email == user_data.email)).first()
    if not user or not verify_password(user_data.password, user.password):
        raise HTTPException(status_code=401, detail="Invalid credentials")
    access_token = create_access_token(data={"sub": str(user.id)})
    return {
        "message": "Login successful",
        "user_id": user.id,
        "access_token": access_token,
        "token_type": "bearer",
    }


@router.get("/profile")
def get_user_profile(current_user: User = Depends(get_current_user)):
    """Get the current user's profile information"""
    return {
        "username": current_user.username,
        "name": current_user.username,  # Just use username since name isn't in the model
        "email": current_user.email,
        "phone": current_user.phone_number,
    }


@router.put("/update")
def update_user_profile(
    user_data: dict, current_user: User = Depends(get_current_user), session: Session = Depends(get_session)
):
    """Update the current user's profile information"""
    # Update user fields
    if "username" in user_data:
        current_user.username = user_data["username"]
    if "email" in user_data:
        current_user.email = user_data["email"]
    if "phone" in user_data:
        current_user.phone_number = user_data["phone"]
    
    session.add(current_user)
    session.commit()
    session.refresh(current_user)
    
    return {"message": "Profile updated successfully"}
